import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Form, notification, Modal, Spin, Tabs, Typography, Popconfirm, Button } from 'antd'
import { CheckCircleOutlined, StopOutlined, SwapOutlined, DeleteOutlined, BorderOutlined } from '@ant-design/icons'

import Api from '@/api'
import Utils from '@/utils/utils.js'
import { formatSearch, joinMainSearchkey } from '@/utils/utils-custom.js'
import SettingUtils from './utils.jsx'
import asyncComponent from '@/utils/asyncComponent'
import DataSource from './datasource'
import MinView from '@/assets/img/minview.png'
import './index.scss'

const { TabPane } = Tabs
const { Paragraph } = Typography
const CodeMirror = asyncComponent(() => import('@/templates/zshare/codemirror'))
const CustomScript = asyncComponent(() => import('@/templates/zshare/customscript'))

class SettingForm extends Component {
  static propTpyes = {
    menu: PropTypes.object,         // 菜单信息
    config: PropTypes.object,       // 页面配置信息
    search: PropTypes.array,        // 搜索条件
    updRecord: PropTypes.func
  }

  state = {
    formlist: [],
    btnloading: false,
    activeKey: 'setting',
    searches: [],
    arr_field: '',
    setting: null,
    defaultSql: '',
    visible: false,
    script: null,
    scriptValue: '',
    status: {}
  }

  UNSAFE_componentWillMount() {
    const { config, search } = this.props

    let _setting = fromJS(config.setting).toJS()
    let _scripts = _setting.scripts || []

    _setting.default = _setting.default || 'true'            // 默认sql
    _setting.sysInterface = _setting.sysInterface || 'false' // 是否为系统接口
    _setting.interface = _setting.sysInterface === 'true' ? (window.GLOB.mainSystemApi || '') : (_setting.interface || '')

    // 显示列字段，用于查询
    let columns = []
    let arr_field = []
    config.columns.forEach(col => {
      if (col.field) {
        columns.push({
          value: col.field,
          text: col.label
        })
        arr_field.push(col.field)
      }
      if (col.nameField) { // 链接的名称字段
        arr_field.push(col.nameField)
      }
    })

    if (_setting.primaryKey && !arr_field.includes(_setting.primaryKey)) {
      _setting.primaryKey = ''
    }
    if (!_setting.primaryKey && arr_field.length > 0) {
      arr_field.forEach(field => {
        if (field.toLowerCase() === 'id') {
          _setting.primaryKey = field
        }
      })
    }

    let status = fromJS(_setting).toJS()

    // if (window.GLOB.funcs && window.GLOB.funcs.length > 0) {
    //   window.GLOB.funcs.forEach(m => {
    //     let reg = new RegExp(`\\/\\*\\$ex@${m.func_code}-begin\\*\\/[\\s\\S]+\\/\\*@ex\\$-end\\*\\/`, 'ig')
    //     _scripts.forEach(item => {
    //       item.sql = item.sql.replace(reg, `$ex@${m.func_code}@ex$`)
    //     })
    //     if (_setting.dataresource) {
    //       _setting.dataresource = _setting.dataresource.replace(reg, `$ex@${m.func_code}@ex$`)
    //     }
    //   })
    // }

    this.setState({
      setting: _setting,
      searches: formatSearch(search),
      arr_field: arr_field.join(','),
      columns: columns,
      scripts: _scripts,
      status
    })
  }

  componentDidMount () {
    this.props.updRecord(this.state.status)
  }

  handleConfirm = (trigger) => {
    const { activeKey, setting, scripts } = this.state

    if (this.scriptsForm && this.scriptsForm.props.form.getFieldValue('sql') && !/^\s+$/.test(this.scriptsForm.props.form.getFieldValue('sql'))) {
      notification.warning({
        top: 92,
        message: '存在未保存脚本，请点击确定保存，或点击取消放弃修改！',
        duration: 5
      })
      return Promise.reject()
    }

    if (trigger) {
      this.setState({loading: true})
    }

    // 表单提交时检查输入值是否正确
    if (activeKey === 'setting') {
      return new Promise((resolve, reject) => {
        this.settingForm.handleConfirm().then(res => {
          this.setState({
            setting: res
          }, () => {
            this.sqlverify(() => {
              this.setState({loading: false})
              resolve({...res, scripts})
            }, () => {
              this.setState({loading: false})
              reject()
            }, 'submit')
          })
        }, () => {
          this.setState({loading: false})
          reject()
        })
      })
    } else if (activeKey === 'scripts') {
      return new Promise((resolve, reject) => {
        this.sqlverify(() => {
          this.setState({loading: false})
          resolve({...setting, scripts})
        }, () => {
          this.setState({loading: false})
          reject()
        }, 'submit')
      })
    } else {
      this.setState({loading: false})
      return new Promise((resolve) => {
        resolve({...setting, scripts})
      })
    }
  }

  sqlverify = (_resolve, _reject, type, uscripts) => {
    const { config } = this.props
    const { setting, scripts, arr_field, searches } = this.state

    if (setting.interType !== 'system') { // 不使用系统接口时，不需要sql验证
      _resolve()
      return
    }

    let _scripts = []
    if (uscripts) { // 过滤禁用的脚本
      _scripts = uscripts.filter(script => script.status !== 'false')
    } else {
      _scripts = scripts.filter(script => script.status !== 'false')
    }

    if (type === 'setting' && setting.default === 'false') {
      _resolve()
    } else if (type === 'scripts' && _scripts.length === 0) {
      _resolve()
    } else { // type 为 submit 、 verify ，以及其他需要验证的场景
      let r = SettingUtils.getDebugSql(setting, _scripts, arr_field, searches, config.urlFields)

      if (r.errors) {
        notification.warning({
          top: 92,
          message: r.errors,
          duration: 5
        })
      }
      
      Api.sDebug(r.sql).then(result => {
        if (result.status || result.ErrCode === '-2') {
          _resolve()
        } else {
          _reject()
          Modal.error({
            title: result.message
          })
        }
      })
    }
  }

  // 标签切换
  changeTab = (val) => {
    const { activeKey, searches, arr_field } = this.state

    if (this.scriptsForm && this.scriptsForm.props.form.getFieldValue('sql') && !/^\s+$/.test(this.scriptsForm.props.form.getFieldValue('sql'))) {
      notification.warning({
        top: 92,
        message: '存在未保存的脚本！',
        duration: 5
      })
    }

    if (activeKey === 'setting') {
      let _defaultSql = ''
      this.settingForm.handleConfirm().then(res => {
        if (res.dataresource) {
          let _dataresource = res.dataresource

          let _search = joinMainSearchkey(searches)

          if (/\s/.test(_dataresource)) {
            _dataresource = '(' + _dataresource + ') tb'
          }

          _defaultSql = `select top @pageSize@ ${arr_field} from (select ${arr_field} ,ROW_NUMBER() over(order by @orderBy@) as rows from ${_dataresource} ${_search}) tmptable where rows > (@pageSize@ * (@pageIndex@ - 1)) order by tmptable.rows`
        }
        
        this.setState({
          setting: res,
          defaultSql: _defaultSql
        }, () => {
          this.setState({loading: true})
          this.sqlverify(() => { // 验证成功
            this.setState({
              activeKey: val,
              loading: false
            })
          }, () => {             // 验证失败
            this.setState({
              activeKey: val,
              loading: false
            })
          }, activeKey)
        })
      })
    } else if (activeKey === 'scripts') {
      this.setState({loading: true})
      this.sqlverify(() => { // 验证成功
        this.setState({
          activeKey: val,
          loading: false
        })
      }, () => {             // 验证失败
        this.setState({
          activeKey: val,
          loading: false
        })
      }, activeKey)
    } else {
      this.setState({
        activeKey: val
      })
    }
  }

  // 自定义脚本修改
  scriptsChange = (scripts) => {
    return new Promise((resolve, reject) => {
      this.sqlverify(resolve, reject, 'verify', scripts)
    })
  }

  triggerConfirm = () => {
    const { script, scriptValue, scripts } = this.state
    let _scripts = fromJS(scripts).toJS()

    if (!scriptValue) {
      notification.warning({
        top: 92,
        message: '请输入sql!',
        duration: 5
      })
      return
    }

    if (script) {
      _scripts = _scripts.map(item => {
        if (script.uuid === item.uuid) {
          item.sql = scriptValue
        }
        return item
      })
    } else {
      let _script = {
        uuid: Utils.getuuid(),
        sql: scriptValue,
        $index: _scripts.length + 1,
        status: 'true'
      }

      _scripts.push(_script)
    }

    this.setState({loading: true})

    this.sqlverify(() => {this.setState({scripts: _scripts, script: null, scriptValue: '', loading: false})}, () => {this.setState({loading: false})}, 'verify', _scripts)
  }

  // 自定义脚本更新
  scriptsUpdate = (scripts) => {
    this.setState({scripts})
  }

  handleDelete = (item) => {
    const { script, scripts } = this.state

    if (script && script.uuid === item.uuid) {
      this.setState({script: null})
    }
    this.setState({scripts: scripts.filter(cell => cell.uuid !== item.uuid)})
  }

  handleStatus = (item) => {
    item.status = item.status === 'false' ? 'true' : 'false'

    this.setState({scripts: this.state.scripts.map(cell => {
      if (cell.uuid === item.uuid) {
        return item
      }
      return cell
    })})
  }

  updateStatus = (status) => {
    let _status = {...this.state.status, ...status}
    this.setState({status: _status})
    this.props.updRecord(_status)
  }

  render() {
    const { config, menu } = this.props
    const { loading, activeKey, setting, defaultSql, columns, scripts, status, visible, script, scriptValue } = this.state

    return (
      <div className="model-table-setting-form-box">
        {loading && <Spin size="large" />}
        <Tabs activeKey={activeKey} onChange={this.changeTab}>
          <TabPane tab="数据源" key="setting">
            <DataSource
              menu={menu}
              config={config}
              columns={columns}
              setting={setting}
              scripts={scripts}
              updateStatus={this.updateStatus}
              wrappedComponentRef={(inst) => this.settingForm = inst}
            />
          </TabPane>
          <TabPane tab={
            <span>
              自定义脚本
              {scripts.length ? <span className="count-tip">{scripts.length}</span> : null}
            </span>
          } disabled={status.interType !== 'system'} key="scripts" id="mk-setting-scripts">
            {scripts.length ? <BorderOutlined className="full-scripts" onClick={() => {
              if (this.scriptsForm && (this.scriptsForm.state.editItem || (this.scriptsForm.props.form.getFieldValue('sql') && !/^\s+$/.test(this.scriptsForm.props.form.getFieldValue('sql'))))) {
                notification.warning({
                  top: 92,
                  message: '请保存自定义脚本！',
                  duration: 5
                })
                return
              }
              this.setState({visible: true, script: null, scriptValue: ''})
            }}/> : null}
            <CustomScript
              setting={setting}
              scripts={scripts}
              defaultSql={defaultSql}
              urlFields={config.urlFields}
              searches={this.state.searches}
              scriptsChange={this.scriptsChange}
              scriptsUpdate={this.scriptsUpdate}
              wrappedComponentRef={(inst) => this.scriptsForm = inst}
            />
          </TabPane>
        </Tabs>
        <Modal
          wrapClassName="model-custom-table-scripts-modal"
          title="自定义脚本"
          visible={visible}
          width={'95vw'}
          maskClosable={false}
          destroyOnClose
        >
          <img className="unfull-scripts" src={MinView} onClick={() => this.setState({visible: false, script: null})} alt=""/>
          <div className="script-table-wrap">
            {scripts.map(item => {
              let title = item.sql.match(/^\s*\/\*.+\*\//)
              title = title && title[0] ? title[0] : ''
              let _text = title ? item.sql.replace(title, '') : item.sql

              if (item.status === 'false') {
                return (
                  <div className="script-item" key={item.uuid}>
                    <div style={{cursor: 'not-allowed'}}>
                      {title ? <div style={{color: '#a50', whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis'}}>{title}</div> : null}
                      <Paragraph copyable={{ text: item.sql }} ellipsis={{ rows: 4 }}>{_text}</Paragraph>
                      <div>
                        <span style={{color: '#ff4d4f', marginLeft: '20px'}}>
                          禁用
                          <StopOutlined style={{marginLeft: '5px'}} />
                        </span>
                      </div>
                    </div>
                    <div style={{height: '24px'}}></div>
                  </div>
                )
              } else {
                return (
                  <div className={'script-item ' + (script && script.uuid === item.uuid ? 'active' : '') } key={item.uuid}>
                    <div style={{cursor: 'pointer'}} onClick={() => {
                      this.setState({script: item, scriptValue: item.sql})
                    }}>
                      {title ? <div style={{color: '#a50', whiteSpace: 'nowrap', overflow: 'hidden', textOverflow: 'ellipsis'}}>{title}</div> : null}
                      <Paragraph copyable={{ text: item.sql }} ellipsis={{ rows: 4 }}>{_text}</Paragraph>
                      <div>
                        <span style={{color: '#26C281', marginLeft: '20px'}}>
                          启用
                          <CheckCircleOutlined style={{marginLeft: '5px'}}/>
                        </span>
                      </div>
                    </div>
                    <div style={{textAlign: 'right'}}>
                      <span className="operation-btn" onClick={() => this.handleStatus(item)} style={{color: '#8E44AD'}}><SwapOutlined /></span>
                      <Popconfirm
                        overlayClassName="popover-confirm"
                        title="确定删除吗?"
                        onConfirm={() => this.handleDelete(item)
                      }>
                        <span className="operation-btn" style={{color: '#ff4d4f'}}><DeleteOutlined /></span>
                      </Popconfirm>
                    </div>
                  </div>
                )
              }
            })}
          </div>
          <div className="script-button">
            <Button onClick={this.triggerConfirm} loading={loading} className="mk-green" style={{marginBottom: 15, marginLeft: 40}}>
              {script ? '保存' : '添加'}
            </Button>
            <Button onClick={() => {this.setState({script: null, scriptValue: ''})}} style={{marginBottom: 15, marginLeft: 10}}>
              取消
            </Button>
          </div>
          <CodeMirror value={scriptValue} onChange={(val) => {this.setState({scriptValue: val})}}></CodeMirror>
        </Modal>
      </div>
    )
  }
}

export default Form.create()(SettingForm)